'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = imsBootstrapper;

require('event-source-polyfill');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _reactRouter = require('react-router');

var _reactRouterRedux = require('react-router-redux');

var _imsSharedCore = require('ims-shared-core');

var _App = require('../components/App/');

var _App2 = _interopRequireDefault(_App);

var _auth = require('../utils/auth');

var _cookies = require('../utils/cookies');

var _window = require('../utils/window');

var _PackageInformation = require('../PackageInformation');

var _PackageInformation2 = _interopRequireDefault(_PackageInformation);

var _stateInitializer = require('./stateInitializer');

var _createStore = require('./store/createStore');

var _createStore2 = _interopRequireDefault(_createStore);

var _routing = require('./modules/routing');

var _universal = require('./modules/universal');

var _system = require('./modules/system');

var _packages = require('./modules/packages');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function imsBootstrapper() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var opts = validateOptions(options);

  // ========================================================
  // Configure PackageInformation
  // ========================================================
  window.versionInfo = opts.config;
  _PackageInformation2.default.configure(window.versionInfo);

  // ========================================================
  // Load HUB URL from cookies
  // ========================================================
  window.__HUB_URL__ = (0, _cookies.getCookie)(_imsSharedCore.Constants.Cookies.HubUrl);

  // ========================================================
  // Set window variable for development mode
  // ========================================================
  window.__DEV__ = opts.isDev;

  // ========================================================
  // If no guest access is enabled and there is no
  // refresh token redirect to login
  // ========================================================
  if (opts.noGuestAccess) {
    var rt = (0, _cookies.getCookie)(_imsSharedCore.Constants.Cookies.RefreshToken);
    if (!rt) {
      (0, _window.setLocation)((0, _auth.getAuthorizeUrl)(window.location.pathname));
      return;
    }
  }

  // ========================================================
  // Store and History Instantiation
  // ========================================================
  (0, _stateInitializer.loadInitialState)(opts.stateInitializer).then(completeInitialization(opts));
}

function validateOptions(options) {
  if (!_lodash2.default.isObjectLike(options)) {
    throw new TypeError('options should be an object');
  }

  var lOpts = Object.assign({}, options);

  if (!_lodash2.default.isObjectLike(lOpts.config)) {
    throw new TypeError('options.config is required and should be an object');
  }

  if (lOpts.configureHMR && !_lodash2.default.isFunction(lOpts.configureHMR)) {
    throw new TypeError('options.configureHMR must be a function');
  }

  if (lOpts.stateInitializer) {
    if (!_lodash2.default.isObjectLike(lOpts.stateInitializer)) {
      throw new TypeError('options.stateInitializer should be an object');
    }

    if (!_lodash2.default.isString(lOpts.stateInitializer.url)) {
      throw new TypeError('options.stateInitializer.url is required and should be a string');
    }
  }

  if (lOpts.reducers && !_lodash2.default.isObjectLike(lOpts.reducers)) {
    throw new TypeError('options.reducers should be an object');
  }

  if (!_lodash2.default.isFunction(lOpts.routes)) {
    throw new TypeError('options.routes is required and should be a function');
  }

  if (lOpts.sagas && !_lodash2.default.isObjectLike(lOpts.sagas)) {
    throw new TypeError('options.sagas should be an object');
  }

  if (lOpts.tracking && !_lodash2.default.isObjectLike(lOpts.tracking)) {
    throw new TypeError('options.tracking should be an object');
  }

  if (lOpts.onAppWillMount && !_lodash2.default.isFunction(lOpts.onAppWillMount)) {
    throw new TypeError('options.onAppWillMount should be a function');
  }

  if (!_lodash2.default.isBoolean(lOpts.isDev)) {
    lOpts.isDev = false;
  }

  if (!_lodash2.default.isBoolean(lOpts.noGuestAccess)) {
    lOpts.noGuestAccess = true;
  }

  return lOpts;
}

// Load packages access from the hub
function updatePackages(store) {
  (0, _stateInitializer.loadPackagesFromHub)().then(function (result) {
    if (result.success) {
      store.dispatch((0, _packages.createPackageState)(result.packages));

      // Badge support
      for (var i = 0; i < result.packages.length; i++) {
        (0, _stateInitializer.getPackageBadgeCount)(result.packages[i]).then(function (countResponse) {
          store.dispatch((0, _packages.setPackageBadgeCount)(countResponse.id, countResponse.count));
        });
      }
    }
  });
}

function completeInitialization(options) {
  return function (state) {
    // ========================================================
    // Create Store and History
    // ========================================================
    (0, _createStore2.default)(state.initialState, _reactRouter.browserHistory, options).then(function (store) {
      store.dispatch((0, _system.checkClientTime)());
      if (state.fromLocalStorage === true) {
        (0, _stateInitializer.loadInitialStateFromServer)(options.stateInitializer.url).then(function (initialState) {
          if (initialState) {
            store.dispatch((0, _universal.serverInitialState)(initialState));
          }
        });
      }

      window.setTimeout(function () {
        updatePackages(store);
      }, 0);
      window.setInterval(function () {
        updatePackages(store);
      }, 60000);

      var history = (0, _reactRouterRedux.syncHistoryWithStore)(_reactRouter.browserHistory, store, {
        selectLocationState: (0, _routing.selectLocationState)()
      });

      // ========================================================
      // Render Setup
      // ========================================================
      var MOUNT_NODE = document.getElementById('root');

      var render = function render() {
        var routes = options.routes(store);

        var WrapperComponent = options.wrapperComponent || _react2.default.Fragment;
        var appComponent = _react2.default.createElement(
          WrapperComponent,
          null,
          _react2.default.createElement(_App2.default, {
            store: store,
            history: history,
            routes: routes,
            onAppWillMount: options.onAppWillMount
          })
        );

        _reactDom2.default.render(appComponent, MOUNT_NODE);
      };

      if (options.isDev === true) {
        if (module.hot) {
          // Development render functions
          var renderApp = render;
          var renderError = function renderError(error) {
            var RedBox = require('redbox-react').default;

            _reactDom2.default.render(_react2.default.createElement(RedBox, { error: error }), MOUNT_NODE);
          };

          // Wrap render in try/catch
          render = function render() {
            try {
              renderApp();
            } catch (error) {
              renderError(error);
            }
          };

          if (options.configureHMR) {
            options.configureHMR(function () {
              setTimeout(function () {
                _reactDom2.default.unmountComponentAtNode(MOUNT_NODE);
                render();
              });
            });
          }
        }
      }

      // ========================================================
      // Go!
      // ========================================================
      render();
    }).catch(function (err) {
      // A rejection here likely means that you are being redirected to the login page
      console.log(err.message);
    });
  };
}