'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.selectPackages = exports.SET_PACKAGE_BADGE_COUNT = exports.SET_PACKAGES_STATE = undefined;

var _ACTION_HANDLERS;

exports.createPackageState = createPackageState;
exports.setPackageBadgeCount = setPackageBadgeCount;
exports.default = Reducer;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _immutable = require('immutable');

var _reselect = require('reselect');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; } // ------------------------------------
// Imports
// ------------------------------------


// ------------------------------------
// Constants
// ------------------------------------
var SET_PACKAGES_STATE = exports.SET_PACKAGES_STATE = '@@PACKAGES/SET_PACKAGES_STATE';
var SET_PACKAGE_BADGE_COUNT = exports.SET_PACKAGE_BADGE_COUNT = '@@PACKAGES/SET_PACKAGE_BADGE_COUNT';

// ------------------------------------
// Actions
// ------------------------------------
function createPackageState(packages) {
  return {
    type: SET_PACKAGES_STATE,
    packages: packages
  };
}

function setPackageBadgeCount(id, count) {
  return {
    type: SET_PACKAGE_BADGE_COUNT,
    id: id,
    count: count
  };
}

// ------------------------------------
// Action Handlers
// ------------------------------------
var ACTION_HANDLERS = (_ACTION_HANDLERS = {}, _defineProperty(_ACTION_HANDLERS, SET_PACKAGES_STATE, function (state, action) {
  // Prevent zeroing out the badge count during an update
  for (var i = 0; i < state.length; i++) {
    var packageIndex = action.packages.findIndex(function (p) {
      return p.packageId === state[i].packageId;
    });
    if (packageIndex > -1) {
      action.packages[packageIndex].badgeCount = state[i].badgeCount;
    }
  }

  return (0, _immutable.fromJS)(action.packages);
}), _defineProperty(_ACTION_HANDLERS, SET_PACKAGE_BADGE_COUNT, function (state, action) {
  var packages = state.toJS();
  for (var i = 0; i < packages.length; i++) {
    if (packages[i].id === action.id) {
      packages[i].badgeCount = action.count;
    }
  }
  return (0, _immutable.fromJS)(packages);
}), _ACTION_HANDLERS);
// ------------------------------------
// Reducer
// ------------------------------------
var initialState = [];
function Reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _immutable.fromJS)(initialState);
  var action = arguments[1];

  var handler = ACTION_HANDLERS[action.type];

  return handler ? handler(state, action) : state;
}

// ------------------------------------
// Selectors
// ------------------------------------
var selectModuleState = function selectModuleState() {
  return function (state) {
    return state.get('__USER__PACKAGES');
  };
};

var selectPackages = exports.selectPackages = function selectPackages() {
  return (0, _reselect.createSelector)(selectModuleState(), function (mState) {
    return _immutable.List.isList(mState) ? mState.toJS() : [];
  });
};