'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.selectAccessToken = exports.selectTokens = exports.selectCurrentUser = exports.SECURITY_LOGOUT = undefined;

var _ACTION_HANDLERS;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; // ------------------------------------
// Imports
// ------------------------------------


exports.logout = logout;
exports.createSecurityState = createSecurityState;
exports.createPackageSecurityObject = createPackageSecurityObject;
exports.default = Reducer;
exports.logoutSaga = logoutSaga;
exports.updateAccessTokenSaga = updateAccessTokenSaga;
exports.watchLogout = watchLogout;
exports.watchUpdateAccessToken = watchUpdateAccessToken;
exports.rootSaga = rootSaga;

var _reselect = require('reselect');

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

var _immutable = require('immutable');

var _imsSharedCore = require('ims-shared-core');

var _stateInitializer = require('../stateInitializer');

var _auth = require('../../utils/auth');

var _cookies = require('../../utils/cookies');

var _localStorage = require('../../utils/localStorage');

var _request = require('../../utils/request');

var _window = require('../../utils/window');

var _PackageInformation = require('../../PackageInformation');

var _PackageInformation2 = _interopRequireDefault(_PackageInformation);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _marked = /*#__PURE__*/regeneratorRuntime.mark(logoutSaga),
    _marked2 = /*#__PURE__*/regeneratorRuntime.mark(updateAccessTokenSaga),
    _marked3 = /*#__PURE__*/regeneratorRuntime.mark(watchLogout),
    _marked4 = /*#__PURE__*/regeneratorRuntime.mark(watchUpdateAccessToken),
    _marked5 = /*#__PURE__*/regeneratorRuntime.mark(rootSaga);

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// ------------------------------------
// Constants
// ------------------------------------
var SECURITY_LOGOUT = exports.SECURITY_LOGOUT = '@@security/LOGOUT';

// ------------------------------------
// Actions
// ------------------------------------
function logout() {
  return {
    type: SECURITY_LOGOUT
  };
}

// ------------------------------------
// Action Handlers
// ------------------------------------
function createSecurityState(accessToken, expires) {
  return {
    currentUser: decodeAccessToken(accessToken),
    tokens: {
      accessToken: accessToken,
      expires: expires
    }
  };
}

function createPackageSecurityObject(packageInfo) {
  if (packageInfo && packageInfo.accessToken && packageInfo.accessToken.token) {
    return _extends({
      userInfo: decodeAccessToken(packageInfo.accessToken.token),
      expires: packageInfo.accessToken.expires
    }, packageInfo);
  } else {
    return _extends({}, packageInfo);
  }
}

function decodeAccessToken(accessToken) {
  var decoded = (0, _auth.decodeJWT)(accessToken);
  var permHandler = new _imsSharedCore.PermissionHandler({ package: { id: _PackageInformation2.default.packageId } });
  decoded.permissions = permHandler.createPermissionsArrayFromStringsArray(decoded.claims);
  return decoded;
}

var ACTION_HANDLERS = (_ACTION_HANDLERS = {}, _defineProperty(_ACTION_HANDLERS, _request.UPDATE_ACCESS_TOKEN, function (state, action) {
  var newSecurityState = createSecurityState(action.accessToken, action.expires);

  return state.merge((0, _immutable.fromJS)(newSecurityState));
}), _defineProperty(_ACTION_HANDLERS, _request.UPDATE_PACKAGE_ACCESS_TOKENS, function (state, action) {
  var packages = action.accessTokens.map(function (p) {
    return createPackageSecurityObject(p);
  });
  return state.set('packages', (0, _immutable.fromJS)(packages));
}), _ACTION_HANDLERS);

// ------------------------------------
// Reducer
// ------------------------------------
var initialState = {
  currentUser: undefined,
  tokens: {
    refreshToken: undefined,
    accessToken: undefined,
    expires: 0
  },
  packages: []
};
function Reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _immutable.fromJS)(initialState);
  var action = arguments[1];

  var handler = ACTION_HANDLERS[action.type];

  return handler ? handler(state, action) : state;
}

// ------------------------------------
// Sagas
// ------------------------------------
function logoutSaga(action) {
  var authorizeUrl;
  return regeneratorRuntime.wrap(function logoutSaga$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return (0, _effects.call)(_cookies.deleteCookie, _imsSharedCore.Constants.Cookies.RefreshToken);

        case 2:
          _context.next = 4;
          return (0, _effects.call)(_cookies.deleteSpecificCookie, 'SSO', 'nucor-yamato.com');

        case 4:
          _context.next = 6;
          return (0, _effects.call)(_localStorage.removeItems, [_stateInitializer.INITIAL_STATE_STORAGE_KEY, _stateInitializer.SECURITY_STORAGE_KEY]);

        case 6:

          // redirect to the authorizeUrl
          authorizeUrl = (0, _auth.getAuthorizeUrl)();
          _context.next = 9;
          return (0, _effects.call)(_window.setLocation, authorizeUrl);

        case 9:
        case 'end':
          return _context.stop();
      }
    }
  }, _marked, this);
}

function updateAccessTokenSaga(action) {
  var securityState;
  return regeneratorRuntime.wrap(function updateAccessTokenSaga$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.next = 2;
          return (0, _effects.select)(selectModuleState());

        case 2:
          securityState = _context2.sent;
          _context2.next = 5;
          return (0, _effects.call)(_localStorage.setItem, _stateInitializer.SECURITY_STORAGE_KEY, securityState.toJS());

        case 5:
        case 'end':
          return _context2.stop();
      }
    }
  }, _marked2, this);
}

function watchLogout() {
  return regeneratorRuntime.wrap(function watchLogout$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          _context3.next = 2;
          return (0, _reduxSaga.takeEvery)(SECURITY_LOGOUT, logoutSaga);

        case 2:
        case 'end':
          return _context3.stop();
      }
    }
  }, _marked3, this);
}

function watchUpdateAccessToken() {
  return regeneratorRuntime.wrap(function watchUpdateAccessToken$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          _context4.next = 2;
          return (0, _reduxSaga.takeLatest)(_request.UPDATE_ACCESS_TOKEN, updateAccessTokenSaga);

        case 2:
        case 'end':
          return _context4.stop();
      }
    }
  }, _marked4, this);
}

function rootSaga() {
  return regeneratorRuntime.wrap(function rootSaga$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
          _context5.next = 2;
          return (0, _effects.fork)(watchLogout);

        case 2:
          _context5.next = 4;
          return (0, _effects.fork)(watchUpdateAccessToken);

        case 4:
        case 'end':
          return _context5.stop();
      }
    }
  }, _marked5, this);
}

// ------------------------------------
// Selectors
// ------------------------------------
var selectModuleState = function selectModuleState() {
  return function (state) {
    return state.get('security');
  };
};

var selectCurrentUser = exports.selectCurrentUser = function selectCurrentUser() {
  return (0, _reselect.createSelector)(selectModuleState(), function (mState) {
    var currentUser = mState.get('currentUser');
    return _immutable.Map.isMap(currentUser) ? currentUser.toJS() : {};
  });
};

var selectTokens = exports.selectTokens = function selectTokens() {
  return (0, _reselect.createSelector)(selectModuleState(), function (globalState) {
    var tokens = globalState.get('tokens');
    return _immutable.Map.isMap(tokens) ? tokens.toJS() : {};
  });
};

var selectAccessToken = exports.selectAccessToken = function selectAccessToken() {
  return (0, _reselect.createSelector)(selectModuleState(), function (globalState) {
    return globalState.getIn(['tokens', 'accessToken']);
  });
};