'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.selectTooltip = exports.selectIsSaving = exports.selectIsEditing = exports.selectEditTooltip = exports.TOOLTIPS_SAVE_SUCCESS = exports.TOOLTIPS_SAVE_FAILURE = exports.TOOLTIPS_SAVE = exports.TOOLTIPS_FETCH_SUCCESS = exports.TOOLTIPS_FETCH_FAILURE = exports.TOOLTIPS_FETCH = exports.TOOLTIPS_CHANGE_LINK = exports.TOOLTIPS_CHANGE_CONTENT = exports.TOOLTIPS_CANCEL_EDIT = exports.TOOLTIPS_BEGIN_EDIT = undefined;

var _ACTION_HANDLERS;

exports.beginEdit = beginEdit;
exports.cancelEdit = cancelEdit;
exports.changeContent = changeContent;
exports.changeLink = changeLink;
exports.fetchTooltips = fetchTooltips;
exports.fetchTooltipsSuccess = fetchTooltipsSuccess;
exports.fetchTooltipsFailure = fetchTooltipsFailure;
exports.saveTooltip = saveTooltip;
exports.saveTooltipFailure = saveTooltipFailure;
exports.saveTooltipSuccess = saveTooltipSuccess;
exports.default = Reducer;
exports.rootSaga = rootSaga;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _immutable = require('immutable');

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

var _reselect = require('reselect');

var _PackageInformation = require('../../PackageInformation');

var _PackageInformation2 = _interopRequireDefault(_PackageInformation);

var _request = require('../../utils/request');

var _request2 = _interopRequireDefault(_request);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _marked = /*#__PURE__*/regeneratorRuntime.mark(fetchTooltipsSaga),
    _marked2 = /*#__PURE__*/regeneratorRuntime.mark(saveTooltipSaga),
    _marked3 = /*#__PURE__*/regeneratorRuntime.mark(watchFetchTooltips),
    _marked4 = /*#__PURE__*/regeneratorRuntime.mark(watchSaveTooltip),
    _marked5 = /*#__PURE__*/regeneratorRuntime.mark(rootSaga);

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; } // ------------------------------------
// Imports
// ------------------------------------


// ------------------------------------
// Constants
// ------------------------------------
var TOOLTIPS_BEGIN_EDIT = exports.TOOLTIPS_BEGIN_EDIT = '@@TOOLTIPS/BEGIN_EDIT';
var TOOLTIPS_CANCEL_EDIT = exports.TOOLTIPS_CANCEL_EDIT = '@@TOOLTIPS/CANCEL_EDIT';
var TOOLTIPS_CHANGE_CONTENT = exports.TOOLTIPS_CHANGE_CONTENT = '@@TOOLTIPS/CHANGE_CONTENT';
var TOOLTIPS_CHANGE_LINK = exports.TOOLTIPS_CHANGE_LINK = '@@TOOLTIPS/CHANGE_LINK';
var TOOLTIPS_FETCH = exports.TOOLTIPS_FETCH = '@@TOOLTIPS/FETCH';
var TOOLTIPS_FETCH_FAILURE = exports.TOOLTIPS_FETCH_FAILURE = '@@TOOLTIPS/FETCH_FAILURE';
var TOOLTIPS_FETCH_SUCCESS = exports.TOOLTIPS_FETCH_SUCCESS = '@@TOOLTIPS/FETCH_SUCCESS';
var TOOLTIPS_SAVE = exports.TOOLTIPS_SAVE = '@@TOOLTIPS/SAVE';
var TOOLTIPS_SAVE_FAILURE = exports.TOOLTIPS_SAVE_FAILURE = '@@TOOLTIPS/SAVE_FAILURE';
var TOOLTIPS_SAVE_SUCCESS = exports.TOOLTIPS_SAVE_SUCCESS = '@@TOOLTIPS/SAVE_SUCCESS';

// ------------------------------------
// Actions
// ------------------------------------
function beginEdit(tooltip) {
  return {
    type: TOOLTIPS_BEGIN_EDIT,
    tooltip: tooltip
  };
}

function cancelEdit(tooltipId) {
  return {
    type: TOOLTIPS_CANCEL_EDIT,
    tooltipId: tooltipId
  };
}

function changeContent(tooltipId, content) {
  return {
    type: TOOLTIPS_CHANGE_CONTENT,
    tooltipId: tooltipId,
    content: content
  };
}

function changeLink(tooltipId, link) {
  return {
    type: TOOLTIPS_CHANGE_LINK,
    tooltipId: tooltipId,
    link: link
  };
}

function fetchTooltips() {
  return {
    type: TOOLTIPS_FETCH
  };
}

function fetchTooltipsSuccess(tooltips) {
  return {
    type: TOOLTIPS_FETCH_SUCCESS,
    tooltips: tooltips
  };
}

function fetchTooltipsFailure(error) {
  return {
    type: TOOLTIPS_FETCH_FAILURE,
    error: error
  };
}

function saveTooltip(tooltip) {
  return {
    type: TOOLTIPS_SAVE,
    tooltip: tooltip
  };
}

function saveTooltipFailure(tooltipId, error) {
  return {
    type: TOOLTIPS_SAVE_FAILURE,
    tooltipId: tooltipId,
    error: error
  };
}

function saveTooltipSuccess(tooltipId, tooltip) {
  return {
    type: TOOLTIPS_SAVE_SUCCESS,
    tooltip: tooltip
  };
}

// ------------------------------------
// Action Handlers
// ------------------------------------
var ACTION_HANDLERS = (_ACTION_HANDLERS = {}, _defineProperty(_ACTION_HANDLERS, TOOLTIPS_BEGIN_EDIT, function (state, action) {
  return state.setIn(['edit', '' + action.tooltip.tooltipId], (0, _immutable.fromJS)(action.tooltip));
}), _defineProperty(_ACTION_HANDLERS, TOOLTIPS_CANCEL_EDIT, function (state, action) {
  return state.deleteIn(['edit', '' + action.tooltipId]);
}), _defineProperty(_ACTION_HANDLERS, TOOLTIPS_CHANGE_CONTENT, function (state, action) {
  if (state.hasIn(['edit', '' + action.tooltipId])) {
    return state.setIn(['edit', '' + action.tooltipId, 'content'], action.content);
  } else {
    return state;
  }
}), _defineProperty(_ACTION_HANDLERS, TOOLTIPS_CHANGE_LINK, function (state, action) {
  if (state.hasIn(['edit', '' + action.tooltipId])) {
    return state.setIn(['edit', '' + action.tooltipId, 'link'], action.link);
  } else {
    return state;
  }
}), _defineProperty(_ACTION_HANDLERS, TOOLTIPS_FETCH_FAILURE, function (state, action) {
  return state.set('error', action.error);
}), _defineProperty(_ACTION_HANDLERS, TOOLTIPS_FETCH_SUCCESS, function (state, action) {
  var tooltips = action.tooltips.reduce(function (m, t) {
    m[t.tooltipId] = t;
    return m;
  }, {});

  return state.set('error', '').set('items', (0, _immutable.fromJS)(tooltips));
}), _defineProperty(_ACTION_HANDLERS, TOOLTIPS_SAVE, function (state, action) {
  if (state.hasIn(['edit', '' + action.tooltip.tooltipId])) {
    return state.setIn(['edit', '' + action.tooltip.tooltipId, 'isSaving'], true);
  } else {
    return state;
  }
}), _defineProperty(_ACTION_HANDLERS, TOOLTIPS_SAVE_FAILURE, function (state, action) {
  if (state.hasIn(['edit', '' + action.tooltipId])) {
    return state.setIn(['edit', '' + action.tooltipId, 'error'], action.error).setIn(['edit', '' + action.tooltipId, 'isSaving'], false);
  } else {
    return state;
  }
}), _defineProperty(_ACTION_HANDLERS, TOOLTIPS_SAVE_SUCCESS, function (state, action) {
  return state.setIn(['items', '' + action.tooltip.tooltipId], (0, _immutable.fromJS)(action.tooltip)).deleteIn(['edit', '' + action.tooltip.tooltipId]);
}), _ACTION_HANDLERS);

// ------------------------------------
// Reducer
// ------------------------------------
var initialState = {
  error: '',
  items: {},
  edit: {}
};

function Reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _immutable.fromJS)(initialState);
  var action = arguments[1];

  var handler = ACTION_HANDLERS[action.type];

  return handler ? handler(state, action) : state;
}

// ------------------------------------
// Sagas
// ------------------------------------
function fetchTooltipsSaga() {
  var hubUrl, response;
  return regeneratorRuntime.wrap(function fetchTooltipsSaga$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.prev = 0;
          hubUrl = window.__HUB_URL__;

          if (hubUrl) {
            _context.next = 6;
            break;
          }

          _context.next = 5;
          return (0, _effects.put)(fetchTooltipsFailure('Hub URL not defined'));

        case 5:
          return _context.abrupt('return');

        case 6:
          _context.next = 8;
          return (0, _effects.call)(_request2.default, window.__HUB_URL__ + '/api/packages/' + _PackageInformation2.default.packageId + '/tooltips');

        case 8:
          response = _context.sent;

          if (!(response.success === true)) {
            _context.next = 14;
            break;
          }

          _context.next = 12;
          return (0, _effects.put)(fetchTooltipsSuccess(response.results));

        case 12:
          _context.next = 16;
          break;

        case 14:
          _context.next = 16;
          return (0, _effects.put)(fetchTooltipsFailure(response.message));

        case 16:
          _context.next = 22;
          break;

        case 18:
          _context.prev = 18;
          _context.t0 = _context['catch'](0);
          _context.next = 22;
          return (0, _effects.put)(fetchTooltipsFailure(_context.t0.message));

        case 22:
        case 'end':
          return _context.stop();
      }
    }
  }, _marked, this, [[0, 18]]);
}

function saveTooltipSaga(action) {
  var hubUrl, reqOpts, url, response;
  return regeneratorRuntime.wrap(function saveTooltipSaga$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.prev = 0;
          hubUrl = window.__HUB_URL__;

          if (hubUrl) {
            _context2.next = 6;
            break;
          }

          _context2.next = 5;
          return (0, _effects.put)(saveTooltipFailure(action.tooltip.tooltipId, 'Hub URL not defined'));

        case 5:
          return _context2.abrupt('return');

        case 6:
          reqOpts = {
            method: 'POST',
            body: JSON.stringify(_lodash2.default.pickBy(action.tooltip, function (x) {
              return x !== null;
            }))
          };
          url = window.__HUB_URL__ + '/api/packages/' + _PackageInformation2.default.packageId + '/tooltips';
          _context2.next = 10;
          return (0, _effects.call)(_request2.default, url, reqOpts);

        case 10:
          response = _context2.sent;

          if (!(response.success === true)) {
            _context2.next = 16;
            break;
          }

          _context2.next = 14;
          return (0, _effects.put)(saveTooltipSuccess(response.result.tooltipId, response.result));

        case 14:
          _context2.next = 18;
          break;

        case 16:
          _context2.next = 18;
          return (0, _effects.put)(saveTooltipFailure(action.tooltip.tooltipId, response.message));

        case 18:
          _context2.next = 24;
          break;

        case 20:
          _context2.prev = 20;
          _context2.t0 = _context2['catch'](0);
          _context2.next = 24;
          return (0, _effects.put)(saveTooltipFailure(action.tooltip.tooltipId, _context2.t0.message));

        case 24:
        case 'end':
          return _context2.stop();
      }
    }
  }, _marked2, this, [[0, 20]]);
}

function watchFetchTooltips() {
  return regeneratorRuntime.wrap(function watchFetchTooltips$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          _context3.next = 2;
          return (0, _reduxSaga.takeLatest)(TOOLTIPS_FETCH, fetchTooltipsSaga);

        case 2:
        case 'end':
          return _context3.stop();
      }
    }
  }, _marked3, this);
}

function watchSaveTooltip() {
  return regeneratorRuntime.wrap(function watchSaveTooltip$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          _context4.next = 2;
          return (0, _reduxSaga.takeEvery)(TOOLTIPS_SAVE, saveTooltipSaga);

        case 2:
        case 'end':
          return _context4.stop();
      }
    }
  }, _marked4, this);
}

function rootSaga() {
  return regeneratorRuntime.wrap(function rootSaga$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
          _context5.next = 2;
          return (0, _effects.fork)(watchFetchTooltips);

        case 2:
          _context5.next = 4;
          return (0, _effects.fork)(watchSaveTooltip);

        case 4:
        case 'end':
          return _context5.stop();
      }
    }
  }, _marked5, this);
}

// ------------------------------------
// Selectors
// ------------------------------------
var selectModuleState = function selectModuleState() {
  return function (state) {
    return state.get('tooltips');
  };
};

var selectEditTooltip = exports.selectEditTooltip = function selectEditTooltip(tooltipId) {
  return (0, _reselect.createSelector)(selectModuleState(), function (mState) {
    var editTooltip = mState.getIn(['edit', '' + tooltipId]);
    return _immutable.Map.isMap(editTooltip) ? editTooltip.toJS() : undefined;
  });
};

var selectIsEditing = exports.selectIsEditing = function selectIsEditing(tooltipId) {
  return (0, _reselect.createSelector)(selectModuleState(), function (mState) {
    return mState.hasIn(['edit', '' + tooltipId]);
  });
};

var selectIsSaving = exports.selectIsSaving = function selectIsSaving(tooltipId) {
  return (0, _reselect.createSelector)(selectModuleState(), function (mState) {
    return mState.getIn(['edit', '' + tooltipId, 'isSaving']) || false;
  });
};

var selectTooltip = exports.selectTooltip = function selectTooltip(tooltipId) {
  return (0, _reselect.createSelector)(selectModuleState(), function (mState) {
    var tooltip = mState.getIn(['items', '' + tooltipId]);
    return _immutable.Map.isMap(tooltip) ? tooltip.toJS() : makeTooltip(tooltipId);
  });
};

function makeTooltip(tooltipId) {
  return {
    packageId: _PackageInformation2.default.packageId,
    tooltipId: tooltipId,
    content: '',
    link: ''
  };
}