'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SECURITY_STORAGE_KEY = exports.INITIAL_STATE_STORAGE_KEY = undefined;
exports.loadInitialState = loadInitialState;
exports.loadInitialStateFromServer = loadInitialStateFromServer;
exports.loadPackagesFromHub = loadPackagesFromHub;
exports.getPackageBadgeCount = getPackageBadgeCount;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _isomorphicFetch = require('isomorphic-fetch');

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

var _PackageInformation = require('../PackageInformation');

var _PackageInformation2 = _interopRequireDefault(_PackageInformation);

var _localStorage = require('../utils/localStorage');

var _request = require('../utils/request');

var _request2 = _interopRequireDefault(_request);

var _security = require('./modules/security');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var INITIAL_STATE_STORAGE_KEY = exports.INITIAL_STATE_STORAGE_KEY = 'IMS-initialState';
var SECURITY_STORAGE_KEY = exports.SECURITY_STORAGE_KEY = 'IMS-security';

function loadInitialState(opts) {
  return loadInitialStateFromLocalStorage().then(function (initialState) {
    if (initialState) {
      return { fromLocalStorage: true, initialState: initialState };
    } else {
      return loadInitialStateFromServer(opts.url).then(function (initialState) {
        return { fromLocalStorage: false, initialState: initialState };
      });
    }
  });
}

function loadInitialStateFromLocalStorage() {
  return (0, _localStorage.getItems)([INITIAL_STATE_STORAGE_KEY, SECURITY_STORAGE_KEY]).then(function (storageData) {
    if (_lodash2.default.isObjectLike(storageData) && _lodash2.default.isObjectLike(storageData[INITIAL_STATE_STORAGE_KEY]) && _lodash2.default.isObjectLike(storageData[SECURITY_STORAGE_KEY]) && _lodash2.default.isObjectLike(storageData[SECURITY_STORAGE_KEY].currentUser)) {
      return Object.assign({}, storageData[INITIAL_STATE_STORAGE_KEY], {
        security: storageData[SECURITY_STORAGE_KEY]
      });
    } else {
      return undefined;
    }
  });
}

function createSecurityStateFromResponse(response) {
  if (response.results) {
    var packages = [];
    var securityObject = {};
    for (var i = 0; i < response.results.length; i++) {
      packages.push((0, _security.createPackageSecurityObject)(response.results[i]));
      if (response.results[i].packageId === _PackageInformation2.default.packageId) {
        securityObject = (0, _security.createSecurityState)(response.results[i].accessToken.token, response.results[i].accessToken.expires);
      }
    }
    securityObject.packages = packages;
    return securityObject;
  } else {
    return (0, _security.createSecurityState)(response.accessToken, response.expires);
  }
}

function loadInitialStateFromServer(url) {
  return fetchInitialStateFromServer(url).then(function (serverResponse) {
    if (serverResponse.success !== true || !_lodash2.default.isObjectLike(serverResponse.result) || _lodash2.default.keys(serverResponse.result).length === 0) {
      return undefined;
    }

    return (0, _localStorage.getItem)(SECURITY_STORAGE_KEY).then(function (securityObj) {
      if (!_lodash2.default.isObjectLike(securityObj) || !_lodash2.default.isObjectLike(securityObj.currentUser)) {
        return (0, _request.makeRefreshAccessTokenRequest)().then(function (securityResponse) {
          var _storageData;

          var security = void 0;
          if (securityResponse && securityResponse.success === true) {
            security = createSecurityStateFromResponse(securityResponse);
          }

          var storageData = (_storageData = {}, _defineProperty(_storageData, INITIAL_STATE_STORAGE_KEY, serverResponse.result), _defineProperty(_storageData, SECURITY_STORAGE_KEY, security), _storageData);

          return (0, _localStorage.setItems)(storageData).then(function () {
            return Object.assign({}, serverResponse.result, {
              security: security || {}
            });
          });
        });
      } else {
        return (0, _localStorage.setItem)(INITIAL_STATE_STORAGE_KEY, serverResponse.result).then(function () {
          return serverResponse.result;
        });
      }
    });
  });
}

function loadPackagesFromHub() {
  return (0, _request2.default)('/api/userAccounts/me/packages', { packageId: 'ims.core.administration' });
}

function getPackageBadgeCount(pkgDef) {
  if (pkgDef.badgeUrl) {
    // could just do direct check here to see if it fixes issue
    return (0, _request2.default)(pkgDef.badgeUrl).then(function (result) {
      if (result.success && !isNaN(result.count)) {
        return {
          id: pkgDef.id,
          count: result.count
        };
      } else {
        return {
          id: pkgDef.id,
          count: 0
        };
      }
    }).catch(function () {
      // We never want to hold up the app for a badge request
      // So don't do anything, just return 0
      return {
        id: pkgDef.id,
        count: 0
      };
    });
  } else {
    return Promise.resolve({
      id: pkgDef.id,
      count: 0
    });
  }
}

function fetchInitialStateFromServer(url) {
  return (0, _isomorphicFetch2.default)(url, { credentials: 'same-origin' }).then(_request.parseResponse);
}