'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _redux = require('redux');

var _reactRouterRedux = require('react-router-redux');

var _immutable = require('immutable');

var _sagas = require('./sagas');

var _injectScopedReducer = require('../../utils/injectScopedReducer');

var _request = require('../../utils/request');

var _ActionTracker = require('../../utils/ActionTracker');

var _ActionTracker2 = _interopRequireDefault(_ActionTracker);

var _reducers = require('./reducers');

var _reducers2 = _interopRequireDefault(_reducers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function () {
  var initialState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var history = arguments[1];
  var options = arguments[2];

  var actionTrackerOpts = {
    user: initialState.global ? initialState.global.currentUser : {},
    sessionId: new Date().getTime(),
    interval: 5,
    path: '/events',
    logging: false,
    method: 'POST',
    include: ['@@router/LOCATION_CHANGE'],
    shouldTrackAction: function shouldTrackAction(a) {
      var trackPattern = /.+_SAVE|_SELECT_TREE_NODE|_ADD_ITEM|_CREATE_NEW|_CANCEL/;
      return a['_track'] || trackPattern.test(a.type);
    }
  };

  if (options.tracking) {
    Object.assign(actionTrackerOpts, options.tracking);
  }

  // ======================================================
  // Middleware Configuration
  // ======================================================
  var middleware = [_sagas.sagaMiddleware, (0, _reactRouterRedux.routerMiddleware)(history), (0, _ActionTracker2.default)(actionTrackerOpts)];

  // ======================================================
  // Store Enhancers
  // ======================================================
  var enhancers = [];
  if (options.isDev) {
    var devToolsExtension = window.__REDUX_DEVTOOLS_EXTENSION__;
    if (typeof devToolsExtension === 'function') {
      enhancers.push(devToolsExtension());
    }
  }

  // ======================================================
  // Store Instantiation and HMR Setup
  // ======================================================
  var store = (0, _redux.createStore)((0, _reducers2.default)(options.reducers), (0, _immutable.fromJS)(initialState), _redux.compose.apply(undefined, [_redux.applyMiddleware.apply(undefined, middleware)].concat(enhancers)));
  store.asyncReducers = {};
  // Prepare scoped component injector
  (0, _injectScopedReducer.prepareInjector)(store, _reducers.injectReducer);

  // Initialize global sagas
  (0, _sagas.initializeSagas)(options.sagas);

  if (module.hot) {
    module.hot.accept('./reducers', function () {
      var reducers = require('./reducers').default;
      store.replaceReducer(reducers(options.reducers, store.asyncReducers));
    });
  }

  // Prepare request object
  return (0, _request.prepareRequest)(store, 'security.tokens', 'security.packages').then(function (response) {
    if (!response || !response.success) {
      throw new Error(response.message || 'Unable to prepare request');
    }
    return store;
  });
};