'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (options) {
  var log = function log(msg, arg) {
    if (options.logging) {
      console.log(msg, arg);
    }
  };

  var getSeconds = function getSeconds() {
    return new Date().getTime() / 1000;
  };

  var ACTION_QUEUE = [];
  var TIME_TO_WAIT = options.interval * 1000;
  var QUEUE_TIMEOUT = null;
  var SESSION_START_TIME = getSeconds();

  log('Session started at ', SESSION_START_TIME);

  var hubUrl = (0, _cookies.getCookie)(_imsSharedCore.Constants.Cookies.HubUrl);
  if (!hubUrl) {
    throw new Error('"HUB_URL" cookie has not been set');
  }

  var shouldQueueAction = function shouldQueueAction(action) {
    return options.include && options.include.some(function (i) {
      return i === action.type;
    });
  };

  var sendQueue = function sendQueue() {
    QUEUE_TIMEOUT = null;

    var body = JSON.stringify({
      usr: options.user,
      sid: options.sessionId,
      log: ACTION_QUEUE,
      pid: _PackageInformation2.default.packageId,
      ua: navigator.userAgent,
      scr: {
        w: screen.width,
        h: screen.height
      }
    });
    if (options.logging) {
      log('Sending queue', JSON.stringify(JSON.parse(body), null, 2));
    }

    ACTION_QUEUE.length = 0;

    return (0, _request2.default)('' + hubUrl + (options.path || '/events/track'), {
      body: body,
      method: options.method || 'POST',
      requestOptions: options.requestOptions
    });
  };

  var addPropertiesToAction = function addPropertiesToAction(action) {
    var retVal = {};
    Object.assign(retVal, action);
    retVal['!at'] = new Date();
    retVal['!st'] = getSeconds() - SESSION_START_TIME;
    retVal['!w'] = {
      w: window.innerWidth,
      h: window.innerHeight
    };
    return retVal;
  };

  var queueAction = function queueAction(action) {
    if (options.shouldTrackAction && options.shouldTrackAction(action) || options.include && shouldQueueAction(action)) {
      ACTION_QUEUE.push(addPropertiesToAction(action));
      if (QUEUE_TIMEOUT === null) {
        QUEUE_TIMEOUT = window.setTimeout(sendQueue, TIME_TO_WAIT);
      }
    }
  };

  // Actual middleware
  return function (store) {
    return function (next) {
      return function (action) {
        queueAction(action);
        return next(action);
      };
    };
  };
};

var _cookies = require('./cookies');

var _imsSharedCore = require('ims-shared-core');

var _request = require('./request');

var _request2 = _interopRequireDefault(_request);

var _PackageInformation = require('../PackageInformation');

var _PackageInformation2 = _interopRequireDefault(_PackageInformation);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }